# Copyright (C) 2020 Intel Corporation

import requests
import json
import base64
import urllib.parse
import hashlib  ## python3
import time
import random
import os
from collections import OrderedDict
import argparse
import sys

def main(args):
   appkey = 'di6ik9b9JiYfImUB'

   ark_file = args.ark

   if os.path.basename(ark_file) == "dev93_1_8.ark":
      test_model = "ir"
   else:
      test_model = "elevoc"

   #ark_file = "./fcgi/models/dev93_10.ark"
   f = open(ark_file,"rb")
   file_size = os.path.getsize(ark_file)
   print(file_size)

   number = 0
   bypass = 0
   for i in range(file_size):
      #print(i)
      one = f.read(1)
      if (bypass == 0) and (one == b'\0'):
         bypass = 1

      if one == b'\4':
         number += 1
         if number == 2:
            break

   print("end i=", i)
   f.seek(i+5, 0)

   file_data = f.read(file_size - (i+5))
   print("feature ark size:", len(file_data))
   f.close()

   base64_data = base64.b64encode(file_data)
   time_stamp = int(time.time())

   params = OrderedDict()
   params['app_id'] = 2128571502

   if (test_model == "elevoc"):
      params['stage'] = 'RAW_FORMAT_INIT'   ## indicate which stage: init or inference
      params['model'] = './models/ELE_M.raw'
      params['batch'] = 1
   else:
      params['stage'] = 'IR_FORMAT_INIT_NETWORK'   ## indicate which stage: init or inference
      params['model'] = './models/wsj_dnn5b.xml'
      params['batch'] = 8

   params['device'] = 'GNA_AUTO'
   params['scale_factor'] = 2048

   params['speech'] = base64_data
   params['time_stamp'] = time_stamp

   query_string = urllib.parse.urlencode(params)
   query_string += '&app_key=' + appkey
   m = hashlib.md5(query_string.encode())
   sign = m.hexdigest().upper()
   params['sign'] = sign

   ## notice: must uset http_proxy, otherwise can not get link
   HEADERS = {'Content-Type': 'application/json;charset=utf-8'}
   url= 'http://localhost:8080/cgi-bin/fcgi_speech_1'
   #res = requests.post(url,headers=HEADERS,data=params)
   res = requests.post(url,data=params)
   print(res.text)
   res_json = json.loads(res.text)

   if (test_model != "elevoc"):
      params['stage'] = 'IR_FORMAT_INIT_EXENETWORK'   ## indicate which stage: init or inference
      query_string = urllib.parse.urlencode(params)
      query_string += '&app_key=' + appkey
      m = hashlib.md5(query_string.encode())
      sign = m.hexdigest().upper()
      params['sign'] = sign
      res = requests.post(url,data=params)
      print(res.text)
      res_json = json.loads(res.text)

   params['stage'] = 'INFERENCE'   ## indicate which stage: init or inference
   query_string = urllib.parse.urlencode(params)
   query_string += '&app_key=' + appkey
   m = hashlib.md5(query_string.encode())
   sign = m.hexdigest().upper()
   params['sign'] = sign

   start_time = time.time()
   res = requests.post(url,data=params)
   #print(res.text)
   res_json = json.loads(res.text)

   processing_time = time.time() - start_time

   n = res.text.find("inference result")
   n_1 = res.text.find(":", n)
   n_2 = res.text.find("\n", n)

   infer_res = res.text[n_1 + 1 : n_2]
   raw_infer_data = base64.b64decode(infer_res)

   ark_file = '/tmp/raw_score.ark'
   open(ark_file, 'wb').write(raw_infer_data)

   n = res.text.find("time")
   n_1 = res.text.find(":", n)
   n_2 = res.text.find("}", n)
   fcgi_time = res.text[n_1 + 1 : n_2]

   print("fcgi inference time: ", fcgi_time)
   print("processing time is:", processing_time)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(usage = "it's usage tip.", description = "help info.")
    parser.add_argument("-a", "--ark", default = "input_data/dev93_1_8.ark", help = "the path of ark file")

    args = parser.parse_args()
    main(args)
