#!/usr/bin/env python3

# Copyright (C) 2020 Intel Corporation

import sys
import time
import os
import grpc
import inference_service_pb2
import inference_service_pb2_grpc

def get_speech_data(ark_file):
   f = open(ark_file,"rb")
   file_size = os.path.getsize(ark_file)
   print(file_size)

   number = 0
   bypass = 0
   for i in range(file_size):
      #print(i)
      one = f.read(1)
      if (bypass == 0) and (one == b'\0'):
         bypass = 1

      if one == b'\4':
         number += 1
         if number == 2:
            break

   print("end i=", i)
   f.seek(i+5, 0)

   file_data = f.read(file_size - (i+5))
   print("feature ark size:", len(file_data))
   f.close()

   return file_data

def run(port, device):
    with grpc.insecure_channel('localhost:' + port) as channel:
        stub = inference_service_pb2_grpc.InferenceStub(channel)

        def call_service(input_file, func_name):
            with open(input_file, 'rb') as in_file:
                func = getattr(stub, func_name)
                resp = func(inference_service_pb2.Input(buffer=in_file.read()))
                print(func_name + " result:\n\t" + resp.json)

        speech_data = get_speech_data('input_data/dev93_1_8.ark')
        model_file = 'models/wsj_dnn5b.xml'

        func_name = "SimulationLib"
        func = getattr(stub, func_name)
        resp = func(inference_service_pb2.SimInput(stage=1, model=model_file, batch=8, device=device, scale_factor=2048, speech=speech_data))
        print(func_name + " result:\n\t" + resp.json)

        resp = func(inference_service_pb2.SimInput(stage=2, model=model_file, batch=8, device=device, scale_factor=2048, speech=speech_data))
        print(func_name + " result:\n\t" + resp.json)

        for i in range(1):
           start_time = time.time()
           resp = func(inference_service_pb2.SimInput(stage=3, model=model_file, batch=8, device=device, scale_factor=2048, speech=speech_data))
           processing_time = time.time() - start_time
           print(func_name + " result:\n\t" + resp.json)

           resp_str = resp.json

           n = resp_str.find("time")
           n_1 = resp_str.find(":", n)
           n_2 = resp_str.find("}", n)
           fcgi_time = resp_str[n_1 + 1 : n_2 - 1]

           print("fcgi inference time: ", fcgi_time)
           print("processing time is: ", processing_time)

        ark_file = '/tmp/raw_score.ark'
        open(ark_file, 'wb').write(resp.rawdata)



if __name__ == '__main__':
    if len(sys.argv) < 3:
        port = '8081'
        device = 'CPU'
    else:
        port = sys.argv[1]
        device = sys.argv[2]

    if device == 'CPU' or device == 'GNA_HW':
        print("inference device is:", device)
        run(port, device)
    else:
        print("not a available device")
